;Program compiled by Great Cow BASIC (0.9 5/11/2012)
;Need help? See the GCBASIC forums at http://sourceforge.net/projects/gcbasic/forums,
;check the documentation or email w_cholmondeley at users dot sourceforge dot net.

;********************************************************************************

;Set up the assembler options (Chip type, clock source, other bits and pieces)
 LIST p=12F683, r=DEC
#include <P12F683.inc>
 __CONFIG _INTOSCIO & _WDT_OFF & _MCLRE_OFF

;********************************************************************************

;Set aside memory locations for variables
DELAYTEMP	EQU	112
DELAYTEMP2	EQU	113
SysSTATUS	EQU	127
SysW	EQU	126
SysWaitTemp10US	EQU	117
SysWaitTempMS	EQU	114
SysWaitTempMS_H	EQU	115
SysWaitTempS	EQU	116
ADREADPORT	EQU	32
ADTEMP	EQU	33
ALARM	EQU	34
CURRENT	EQU	35
DELTA	EQU	36
PREV	EQU	37
READAD	EQU	38
SaveDelayTemp	EQU	39
SavePCLATH	EQU	40
SaveSysWaitTemp10US	EQU	41
SysIntOffCount	EQU	42
TMR0PRES	EQU	43
TMR0SOURCE	EQU	44

;********************************************************************************

;Alias variables
ALLANSEL	EQU	ANSEL

;********************************************************************************

;Vectors
	ORG	0
	goto	BASPROGRAMSTART
	ORG	4
Interrupt

;********************************************************************************

;Save Context
	movwf	SysW
	swapf	STATUS,W
	movwf	SysSTATUS
	incf	SysIntOffCount,F
	banksel	STATUS
;Store system variables
	movf	SysWaitTemp10US,W
	movwf	SaveSysWaitTemp10US
	movf	DelayTemp,W
	movwf	SaveDelayTemp
	movf	PCLATH,W
	movwf	SavePCLATH
	clrf	PCLATH
;On Interrupt handlers
	btfss	INTCON,T0IE
	goto	NotT0IF
	btfss	INTCON,T0IF
	goto	NotT0IF
	call	UPDATE
	bcf	INTCON,T0IF
	goto	INTERRUPTDONE
NotT0IF
;User Interrupt routine
INTERRUPTDONE
;Restore Context
;Restore system variables
	movf	SaveSysWaitTemp10US,W
	movwf	SysWaitTemp10US
	movf	SaveDelayTemp,W
	movwf	DelayTemp
	movf	SavePCLATH,W
	movwf	PCLATH
	clrf	SysIntOffCount
	swapf	SysSTATUS,W
	movwf	STATUS
	swapf	SysW,F
	swapf	SysW,W
	retfie

;********************************************************************************

;Start of program memory page 0
	ORG	37
BASPROGRAMSTART
;Call initialisation routines
	call	INITSYS
;Enable interrupts
	bsf	INTCON,GIE
	bsf	INTCON,PEIE
	clrf	SysIntOffCount

;Start of the main program
;This creates an alarm circuit that only fires when the
;light level changes rapidly. A slow change, even from
;total darkness to total brightness has no effect. TIMER0
;is used to force interrupts about 60 times a second.
;A light reading is taken each time and if it has
;changed by more than a certain amount, a rapid change
;has occurred. Use a flashlight for testing.
;----- Settings
;----- Constants
;#define ldr	AN0								;LDR on pin 7 (AN0)
;#define buzzer GPIO.1					;buzzer on pin 6
;#define prescaler PS0_128			;divide-by-128 prescaler
;#define limit = 10						;10 units of light level change
;----- Variables
;dim alarm, delta, current, prev as byte					
;----- Program
;dir GPIO 0b000001						;set up port lines
	movlw	1
	banksel	TRISIO
	movwf	TRISIO
;prev = readAD(ldr)					;initialize to current light level
	banksel	ADREADPORT
	clrf	ADREADPORT
	call	FN_READAD
	movf	READAD,W
	movwf	PREV
;InitTimer0 osc, prescaler		;set up TIMER0
	movlw	1
	movwf	TMR0SOURCE
	movlw	6
	movwf	TMR0PRES
	call	INITTIMER0
;on interrupt Timer0Overflow call update	;enable interrupt
	bsf	INTCON,T0IE
;do
SysDoLoop_S1
;if alarm then							;time for alarm?
	movf	ALARM,F
	btfsc	STATUS,Z
	goto	ENDIF1
;set buzzer on						;yes, beep for a second
	bsf	GPIO,1
;wait 1 S
	movlw	1
	movwf	SysWaitTempS
	call	Delay_S
;set buzzer off
	bcf	GPIO,1
;alarm = false						;shut off now
	clrf	ALARM
;end if	
ENDIF1
;loop												;repeat in perpetuity
	goto	SysDoLoop_S1
SysDoLoop_E1
;----- Interrupt Subroutine
BASPROGRAMEND
	sleep
	goto	BASPROGRAMEND

;********************************************************************************

Delay_10US
D10US_START
	movlw	5
	movwf	DELAYTEMP
DelayUS0
	decfsz	DELAYTEMP,F
	goto	DelayUS0
	nop
	decfsz	SysWaitTemp10US, F
	goto	D10US_START
	return

;********************************************************************************

Delay_MS
	incf	SysWaitTempMS_H, F
DMS_START
	movlw	4
	movwf	DELAYTEMP2
DMS_OUTER
	movlw	165
	movwf	DELAYTEMP
DMS_INNER
	decfsz	DELAYTEMP, F
	goto	DMS_INNER
	decfsz	DELAYTEMP2, F
	goto	DMS_OUTER
	decfsz	SysWaitTempMS, F
	goto	DMS_START
	decfsz	SysWaitTempMS_H, F
	goto	DMS_START
	return

;********************************************************************************

Delay_S
DS_START
	movlw	232
	movwf	SysWaitTempMS
	movlw	3
	movwf	SysWaitTempMS_H
	call	Delay_MS
	decfsz	SysWaitTempS, F
	goto	DS_START
	return

;********************************************************************************

INITSYS
;OSCCON = OSCCON OR b'01110000'
	movlw	112
	banksel	OSCCON
	iorwf	OSCCON,F
;SET ADCON0.ADON OFF
	banksel	ADCON0
	bcf	ADCON0,ADON
;SET ADFM OFF
	bcf	ADCON0,ADFM
;ANSEL = 0
	banksel	ANSEL
	clrf	ANSEL
;CMCON0 = 7
	movlw	7
	banksel	CMCON0
	movwf	CMCON0
;GPIO = 0
	clrf	GPIO
	return

;********************************************************************************

INITTIMER0
;SInitTimer0
	goto	SINITTIMER0

;********************************************************************************

FN_READAD
;Set up for 8 bit
;Perform conversion
;LLReadAD
;Dim AllANSEL As Byte Alias ANSEL
;AllANSEL = 0
	banksel	ALLANSEL
	clrf	ALLANSEL
;ADTemp = ADReadPort + 1
	banksel	ADREADPORT
	incf	ADREADPORT,W
	movwf	ADTEMP
;Set C On
	bsf	STATUS,C
;Do
SysDoLoop_S2
;Rotate AllANSEL Left
	banksel	ALLANSEL
	rlf	ALLANSEL,F
;decfsz ADTemp,F
	banksel	ADTEMP
	decfsz	ADTEMP,F
;Loop
	goto	SysDoLoop_S2
SysDoLoop_E2
;SET ADCS1 OFF
	banksel	ANSEL
	bcf	ANSEL,ADCS1
;SET ADCS0 ON
	bsf	ANSEL,ADCS0
;SET ADCON0.CHS0 OFF
	banksel	ADCON0
	bcf	ADCON0,CHS0
;SET ADCON0.CHS1 OFF
	bcf	ADCON0,CHS1
;IF ADReadPort.0 On Then Set ADCON0.CHS0 On
	btfsc	ADREADPORT,0
	bsf	ADCON0,CHS0
;IF ADReadPort.1 On Then Set ADCON0.CHS1 On
	btfsc	ADREADPORT,1
	bsf	ADCON0,CHS1
;SET ADCON0.ADON ON
	bsf	ADCON0,ADON
;Wait AD_Delay
	movlw	2
	movwf	SysWaitTemp10US
	call	Delay_10US
;SET ADCON0.GO_DONE ON
	bsf	ADCON0,GO_DONE
;Wait While ADCON0.GO_DONE ON
SysWaitLoop1
	btfsc	ADCON0,GO_DONE
	goto	SysWaitLoop1
;SET ADCON0.ADON OFF
	bcf	ADCON0,ADON
;ANSEL = 0
	banksel	ANSEL
	clrf	ANSEL
;Write output
;ReadAD = ADRESH
	banksel	ADRESH
	movf	ADRESH,W
	movwf	READAD
	return

;********************************************************************************

SINITTIMER0
;OPTION_REG = OPTION_REG AND 192
	movlw	192
	banksel	OPTION_REG
	andwf	OPTION_REG,F
;SET OPTION_REG.PSA OFF
	bcf	OPTION_REG,PSA
;if TMR0Source = Osc THEN SET OPTION_REG.T0SE OFF
	banksel	TMR0SOURCE
	decf	TMR0SOURCE,W
	btfss	STATUS, Z
	goto	ENDIF6
	banksel	OPTION_REG
	bcf	OPTION_REG,T0SE
ENDIF6
;if TMR0Source = Ext THEN SET OPTION_REG.T0SE ON
	movlw	2
	banksel	TMR0SOURCE
	subwf	TMR0SOURCE,W
	btfss	STATUS, Z
	goto	ENDIF7
	banksel	OPTION_REG
	bsf	OPTION_REG,T0SE
ENDIF7
;clrwdt
	clrwdt
;OPTION_REG = OPTION_REG OR TMR0Pres
	banksel	OPTION_REG
	movf	OPTION_REG,W
	banksel	TMR0PRES
	iorwf	TMR0PRES,W
	banksel	OPTION_REG
	movwf	OPTION_REG
	banksel	STATUS
	return

;********************************************************************************

UPDATE
;intOff										;disable interrupts
	bcf	INTCON,GIE
	incf	SysIntOffCount,F
;T0IF = 0									;clear the overflow flag
	bcf	INTCON,T0IF
;current = readAD(ldr)			;get 8-bit light reading
	clrf	ADREADPORT
	call	FN_READAD
	movf	READAD,W
	movwf	CURRENT
;if current > prev then		;compute the change
	subwf	PREV,W
	btfsc	STATUS, C
	goto	ELSE2_1
;delta = current - prev
	movf	PREV,W
	subwf	CURRENT,W
	movwf	DELTA
;else
	goto	ENDIF2
ELSE2_1
;delta = prev - current
	movf	CURRENT,W
	subwf	PREV,W
	movwf	DELTA
;end if
ENDIF2
;if delta > limit then			;greater than limit?
	movf	DELTA,W
	sublw	10
	btfsc	STATUS, C
	goto	ENDIF3
;alarm = true
	movlw	255
	movwf	ALARM
;end if
ENDIF3
;prev = current
	movf	CURRENT,W
	movwf	PREV
;intOn											;enable interrupts
	movf	SysIntOffCount,F
	btfss	STATUS,Z
	decf	SysIntOffCount,F
	btfsc	STATUS,Z
	bsf	INTCON,GIE
	return

;********************************************************************************


 END
