;Program compiled by Great Cow BASIC (0.9 5/11/2012)
;Need help? See the GCBASIC forums at http://sourceforge.net/projects/gcbasic/forums,
;check the documentation or email w_cholmondeley at users dot sourceforge dot net.

;********************************************************************************

;Set up the assembler options (Chip type, clock source, other bits and pieces)
 LIST p=12F683, r=DEC
#include <P12F683.inc>
 __CONFIG _INTOSCIO & _WDT_OFF & _MCLRE_OFF

;********************************************************************************

;Set aside memory locations for variables
SysSTATUS	EQU	127
SysW	EQU	126
CLOCK	EQU	32
CURRENT	EQU	33
LSN	EQU	34
MSN	EQU	35
PATTERN	EQU	36
SECONDS	EQU	37
SavePCLATH	EQU	38
SysIntOffCount	EQU	39
SysTemp1	EQU	40
TMR0PRES	EQU	41
TMR0SOURCE	EQU	42

;********************************************************************************

;Alias variables
TIMER0	EQU	TMR0

;********************************************************************************

;Vectors
	ORG	0
	goto	BASPROGRAMSTART
	ORG	4
Interrupt

;********************************************************************************

;Save Context
	movwf	SysW
	swapf	STATUS,W
	movwf	SysSTATUS
	incf	SysIntOffCount,F
	banksel	STATUS
;Store system variables
	movf	PCLATH,W
	movwf	SavePCLATH
	clrf	PCLATH
;On Interrupt handlers
	btfss	INTCON,T0IE
	goto	NotT0IF
	btfss	INTCON,T0IF
	goto	NotT0IF
	call	UPDATE
	bcf	INTCON,T0IF
	goto	INTERRUPTDONE
NotT0IF
;User Interrupt routine
INTERRUPTDONE
;Restore Context
;Restore system variables
	movf	SavePCLATH,W
	movwf	PCLATH
	clrf	SysIntOffCount
	swapf	SysSTATUS,W
	movwf	STATUS
	swapf	SysW,F
	swapf	SysW,W
	retfie

;********************************************************************************

;Start of program memory page 0
	ORG	29
BASPROGRAMSTART
;Call initialisation routines
	call	INITSYS
;Enable interrupts
	bsf	INTCON,GIE
	bsf	INTCON,PEIE
	clrf	SysIntOffCount

;Start of the main program
;This program demonstrates one of the three timers in the
;PIC12F683, TIMER0. It counts 1 through 10 in binary on
;the red LEDs every 1 to 10 seconds, and beeps the buzzer
;at the start of every group of 10. In short, this is a
;simple real time clock. Connect GP0 through GP2, and GP4
;to LED1 through LED4, respectively. Connect GP5 to the
;buzzer. Connct GP3 to the pushbutton. The program uses
;interrupts and TIMER0. Press the pushbutton to start it.
;TIMER0 runs at 8MHz/4/128 = 15625. Allowing two instruction
;cycles for the write to TIMER0, loading a value of
;133 causes the interrupt to occur 125 times a second.
;Remember, this is an RC circuit, not crystal controlled so
;there will be variation from chip to chip. I found that a
;value of 131 gave reasonable results for my particular
;PIC12F683, running just a little bit fast.
;----- Setup
;----- Constants
;#define button GPIO.3				;button on pin 4
;#define buzzer GPIO.5				;buzzer on pin 2
;#define prescaler PS0_128		;divide-by-128 prescaler
;#define timing 131					;tweak to adjust timing
;----- Variables
;dim pattern, msn, lsn, clock, seconds, current as byte
;----- Program
;dir GPIO out	         			;all output except button
	banksel	TRISIO
	clrf	TRISIO
;seconds = 11								;initialize the counters
	movlw	11
	banksel	SECONDS
	movwf	SECONDS
;clock = 0
	clrf	CLOCK
;InitTimer0 osc, prescaler		;set up TIMER0
	movlw	1
	movwf	TMR0SOURCE
	movlw	6
	movwf	TMR0PRES
	call	INITTIMER0
;do													;wait until button pressed
SysDoLoop_S1
;TIMER0 = timing						;keep TIMER0 pinned to initial value
	movlw	131
	movwf	TIMER0
;T0IF = 0									;keep overflow flag cleared
	bcf	INTCON,T0IF
;loop while button = 1
	btfsc	GPIO,3
	goto	SysDoLoop_S1
SysDoLoop_E1
;on interrupt Timer0Overflow call update	;enable interrupt
	bsf	INTCON,T0IE
;do													;main loop
SysDoLoop_S2
;if seconds = 11 then			;only count from 1 to 10
	movlw	11
	subwf	SECONDS,W
	btfss	STATUS, Z
	goto	ENDIF1
;seconds = 1
	movlw	1
	movwf	SECONDS
;end if
ENDIF1
;if seconds = 1 then				;only beep on 1st second
	decf	SECONDS,W
	btfss	STATUS, Z
	goto	ELSE2_1
;set buzzer on
	bsf	GPIO,5
;else
	goto	ENDIF2
ELSE2_1
;set buzzer off
	bcf	GPIO,5
;end if	
ENDIF2
;current = GPIO & 0b00100000	;save buzzer status
	movlw	32
	andwf	GPIO,W
	movwf	CURRENT
;msn = seconds * 2 & 0b00010000	;create LED bit pattern
	bcf	STATUS,C
	rlf	SECONDS,W
	movwf	SysTemp1
	movlw	16
	andwf	SysTemp1,W
	movwf	MSN
;lsn = seconds & 0b00000111
	movlw	7
	andwf	SECONDS,W
	movwf	LSN
;pattern = msn | lsn | current
	movf	MSN,W
	iorwf	LSN,W
	movwf	SysTemp1
	iorwf	CURRENT,W
	movwf	PATTERN
;GPIO = pattern						;update the LEDs
	movwf	GPIO
;loop
	goto	SysDoLoop_S2
SysDoLoop_E2
;----- Interrupt Subroutine
BASPROGRAMEND
	sleep
	goto	BASPROGRAMEND

;********************************************************************************

INITSYS
;OSCCON = OSCCON OR b'01110000'
	movlw	112
	banksel	OSCCON
	iorwf	OSCCON,F
;SET ADCON0.ADON OFF
	banksel	ADCON0
	bcf	ADCON0,ADON
;SET ADFM OFF
	bcf	ADCON0,ADFM
;ANSEL = 0
	banksel	ANSEL
	clrf	ANSEL
;CMCON0 = 7
	movlw	7
	banksel	CMCON0
	movwf	CMCON0
;GPIO = 0
	clrf	GPIO
	return

;********************************************************************************

INITTIMER0
;SInitTimer0
	goto	SINITTIMER0

;********************************************************************************

SINITTIMER0
;OPTION_REG = OPTION_REG AND 192
	movlw	192
	banksel	OPTION_REG
	andwf	OPTION_REG,F
;SET OPTION_REG.PSA OFF
	bcf	OPTION_REG,PSA
;if TMR0Source = Osc THEN SET OPTION_REG.T0SE OFF
	banksel	TMR0SOURCE
	decf	TMR0SOURCE,W
	btfss	STATUS, Z
	goto	ENDIF4
	banksel	OPTION_REG
	bcf	OPTION_REG,T0SE
ENDIF4
;if TMR0Source = Ext THEN SET OPTION_REG.T0SE ON
	movlw	2
	banksel	TMR0SOURCE
	subwf	TMR0SOURCE,W
	btfss	STATUS, Z
	goto	ENDIF5
	banksel	OPTION_REG
	bsf	OPTION_REG,T0SE
ENDIF5
;clrwdt
	clrwdt
;OPTION_REG = OPTION_REG OR TMR0Pres
	banksel	OPTION_REG
	movf	OPTION_REG,W
	banksel	TMR0PRES
	iorwf	TMR0PRES,W
	banksel	OPTION_REG
	movwf	OPTION_REG
	banksel	STATUS
	return

;********************************************************************************

UPDATE
;TIMER0 = timing				;get set for next time
	movlw	131
	movwf	TIMER0
;T0IF = 0							;clear the overflow flag
	bcf	INTCON,T0IF
;clock += 1						;increment clock counter
	incf	CLOCK,F
;if clock = 125 then		;a second has passed
	movlw	125
	subwf	CLOCK,W
	btfss	STATUS, Z
	goto	ENDIF3
;clock = 0
	clrf	CLOCK
;seconds += 1
	incf	SECONDS,F
;end if	
ENDIF3
	return

;********************************************************************************


 END
