;Program compiled by Great Cow BASIC (0.9 5/11/2012)
;Need help? See the GCBASIC forums at http://sourceforge.net/projects/gcbasic/forums,
;check the documentation or email w_cholmondeley at users dot sourceforge dot net.

;********************************************************************************

;Set up the assembler options (Chip type, clock source, other bits and pieces)
 LIST p=12F683, r=DEC
#include <P12F683.inc>
 __CONFIG _INTOSCIO & _WDT_OFF & _MCLRE_OFF

;********************************************************************************

;Set aside memory locations for variables
SysSTATUS	EQU	127
SysW	EQU	126
CLOCK	EQU	32
CURRENT	EQU	33
LSN	EQU	34
MSN	EQU	35
PATTERN	EQU	36
SECONDS	EQU	37
SavePCLATH	EQU	38
SysIntOffCount	EQU	39
SysTemp1	EQU	40
TMR1PRES	EQU	41
TMR1SOURCE	EQU	42
TMR2PRES	EQU	43
TMRNUMBER	EQU	44

;********************************************************************************

;Alias variables
TIMER	EQU	TMR1L
TIMER_H	EQU	TMR1H

;********************************************************************************

;Vectors
	ORG	0
	goto	BASPROGRAMSTART
	ORG	4
Interrupt

;********************************************************************************

;Save Context
	movwf	SysW
	swapf	STATUS,W
	movwf	SysSTATUS
	incf	SysIntOffCount,F
	banksel	STATUS
;Store system variables
	movf	PCLATH,W
	movwf	SavePCLATH
	clrf	PCLATH
;On Interrupt handlers
	banksel	PIE1
	btfss	PIE1,T1IE
	goto	NotT1IF
	banksel	PIR1
	btfss	PIR1,T1IF
	goto	NotT1IF
	call	UPDATE
	bcf	PIR1,T1IF
	goto	INTERRUPTDONE
NotT1IF
;User Interrupt routine
INTERRUPTDONE
;Restore Context
;Restore system variables
	banksel	SAVEPCLATH
	movf	SavePCLATH,W
	movwf	PCLATH
	clrf	SysIntOffCount
	swapf	SysSTATUS,W
	movwf	STATUS
	swapf	SysW,F
	swapf	SysW,W
	retfie

;********************************************************************************

;Start of program memory page 0
	ORG	35
BASPROGRAMSTART
;Call initialisation routines
	call	INITSYS
;Enable interrupts
	bsf	INTCON,GIE
	bsf	INTCON,PEIE
	clrf	SysIntOffCount

;Start of the main program
;This is similar to the previous project but now demonstrates
;TIMER1 which is a 16-bit counter..Connect GP0 through GP2,
;and GP4 to LED1 through LED4, respectively. Connect GP5 to
;the buzzer. Connct GP3 to the pushbutton. The program uses
;interrupts and TIMER1. Press the pushbutton to start it.
;TIMER1 runs at 8MHz/4/8/25000 = 10 Hz. Thus, an interrupt
;occurs 10 times per second.
;Remember, this is an RC circuit, not crystal controlled so
;there may be variation from chip to chip.
;----- Setup
;----- Constants
;#define button GPIO.3				;button on pin 4
;#define buzzer GPIO.5				;buzzer on pin 2
;#define prescaler PS1_8 		;divide-by-8 prescaler
;#define timing 40536				;65536-25000
;----- Variables
;dim pattern, msn, lsn, clock, seconds, current as byte
;dim timer as word alias TMR1H, TMR1L
;----- Program
;dir GPIO out	         			;all output except button
	banksel	TRISIO
	clrf	TRISIO
;seconds = 11								;initialize the counters
	movlw	11
	banksel	SECONDS
	movwf	SECONDS
;clock = 0
	clrf	CLOCK
;timer = timing							;get set for next time
	movlw	88
	movwf	TIMER
	movlw	158
	movwf	TIMER_H
;InitTimer1 osc, prescaler		;set up TIMER1
	movlw	1
	movwf	TMR1SOURCE
	movlw	48
	movwf	TMR1PRES
	call	INITTIMER1
;do													;wait until button pressed
SysDoLoop_S1
;loop while button = 1
	btfsc	GPIO,3
	goto	SysDoLoop_S1
SysDoLoop_E1
;startTimer 1								;start the timer
	movlw	1
	movwf	TMRNUMBER
	call	STARTTIMER
;on interrupt Timer1Overflow call update	;enable interrupt
	banksel	PIE1
	bsf	PIE1,T1IE
;do													;main loop
SysDoLoop_S2
;if seconds = 11 then			;only count from 1 to 10
	movlw	11
	banksel	SECONDS
	subwf	SECONDS,W
	btfss	STATUS, Z
	goto	ENDIF1
;seconds = 1
	movlw	1
	movwf	SECONDS
;end if
ENDIF1
;if seconds = 1 then				;only beep on 1st second
	decf	SECONDS,W
	btfss	STATUS, Z
	goto	ELSE2_1
;set buzzer on
	bsf	GPIO,5
;else
	goto	ENDIF2
ELSE2_1
;set buzzer off
	bcf	GPIO,5
;end if	
ENDIF2
;current = GPIO & 0b00100000	;save buzzer status
	movlw	32
	andwf	GPIO,W
	movwf	CURRENT
;msn = seconds * 2 & 0b00010000	;create LED bit pattern
	bcf	STATUS,C
	rlf	SECONDS,W
	movwf	SysTemp1
	movlw	16
	andwf	SysTemp1,W
	movwf	MSN
;lsn = seconds & 0b00000111
	movlw	7
	andwf	SECONDS,W
	movwf	LSN
;pattern = msn | lsn | current
	movf	MSN,W
	iorwf	LSN,W
	movwf	SysTemp1
	iorwf	CURRENT,W
	movwf	PATTERN
;GPIO = pattern						;update the LEDs
	movwf	GPIO
;loop
	goto	SysDoLoop_S2
SysDoLoop_E2
;----- Interrupt Subroutine
BASPROGRAMEND
	sleep
	goto	BASPROGRAMEND

;********************************************************************************

INITSYS
;OSCCON = OSCCON OR b'01110000'
	movlw	112
	banksel	OSCCON
	iorwf	OSCCON,F
;SET ADCON0.ADON OFF
	banksel	ADCON0
	bcf	ADCON0,ADON
;SET ADFM OFF
	bcf	ADCON0,ADFM
;ANSEL = 0
	banksel	ANSEL
	clrf	ANSEL
;CMCON0 = 7
	movlw	7
	banksel	CMCON0
	movwf	CMCON0
;GPIO = 0
	clrf	GPIO
	return

;********************************************************************************

INITTIMER1
;SInitTimer1
	goto	SINITTIMER1

;********************************************************************************

SINITTIMER1
;T1CON = TMR1Pres
	movf	TMR1PRES,W
	movwf	T1CON
;If TMR1Source = Ext Then
	movlw	2
	subwf	TMR1SOURCE,W
	btfsc	STATUS, Z
;Set TMR1CS On
	bsf	T1CON,TMR1CS
;End If
;If TMR1Source = ExtOsc Then
	movlw	3
	subwf	TMR1SOURCE,W
	btfss	STATUS, Z
	goto	ENDIF7
;Set TMR1CS On
	bsf	T1CON,TMR1CS
;Set T1OSCEN On
	bsf	T1CON,T1OSCEN
;End If
ENDIF7
	return

;********************************************************************************

STARTTIMER
;If TMRNumber = 1 Then
	decf	TMRNUMBER,W
	btfsc	STATUS, Z
;Set TMR1ON On
	bsf	T1CON,TMR1ON
;End If
;If TMRNumber = 2 Then
	movlw	2
	subwf	TMRNUMBER,W
	btfss	STATUS, Z
	goto	ENDIF5
;T2CON = TMR2Pres
	movf	TMR2PRES,W
	movwf	T2CON
;Set TMR2ON On
	bsf	T2CON,TMR2ON
;End If
ENDIF5
	return

;********************************************************************************

UPDATE
;timer = timing				;get set for next time
	movlw	88
	movwf	TIMER
	movlw	158
	movwf	TIMER_H
;T1IF = 0							;clear the overflow flag
	bcf	PIR1,T1IF
;clock += 1						;increment clock counter
	incf	CLOCK,F
;if clock = 10 then		;a second has passed
	movlw	10
	subwf	CLOCK,W
	btfss	STATUS, Z
	goto	ENDIF3
;clock = 0
	clrf	CLOCK
;seconds += 1
	incf	SECONDS,F
;end if	
ENDIF3
	return

;********************************************************************************


 END
