;Program compiled by Great Cow BASIC (0.9 5/11/2012)
;Need help? See the GCBASIC forums at http://sourceforge.net/projects/gcbasic/forums,
;check the documentation or email w_cholmondeley at users dot sourceforge dot net.

;********************************************************************************

;Set up the assembler options (Chip type, clock source, other bits and pieces)
 LIST p=12F683, r=DEC
#include <P12F683.inc>
 __CONFIG _INTOSCIO & _WDT_OFF & _MCLRE_OFF

;********************************************************************************

;Set aside memory locations for variables
DELAYTEMP	EQU	112
SysWaitTemp10US	EQU	117
ADREADPORT	EQU	32
ADTEMP	EQU	33
DOPWM	EQU	34
PWMCHANNEL	EQU	35
PWMDUR	EQU	36
READAD	EQU	37
SOFTPWMCYCLES	EQU	38
SOFTPWMDUTY	EQU	39

;********************************************************************************

;Alias variables
ALLANSEL	EQU	ANSEL

;********************************************************************************

;Vectors
	ORG	0
	goto	BASPROGRAMSTART
	ORG	4
	retfie

;********************************************************************************

;Start of program memory page 0
	ORG	5
BASPROGRAMSTART
;Call initialisation routines
	call	INITSYS

;Start of the main program
;Here's a program which uses the comparator rigged as an
;inverter. Thus, it is possible to generate complementary
;output PWM. This is demonstrated on the bicolor LED.
;Turning the pot will fade the LED from red to yellow to
;green and back again. Connect GP0 to the red element of
;the bicolor LED, and also to GP1, which is the comparator
;input. (Use a multiple to split the signal). Connect GP2
;to the green element. This is the comparator output and
;will be the complement of the PWM signal on GP0. Connect
;GP4 to the potentiometer. Note that software PWM is
;used here.
;The comparator is rigged to perform as a simple inverter,
;a rather neat way to obviate the use of an external
;inverter or transistor. A threshold of 2.5V is used.
;If desired, monitor pins 7 and 5 on a dual channel scope
;to see the complements.
;----- Settings
;----- Constants
;#define PWM_Out1 GPIO.0				;main output, also feeds comparator
;#define compIn GPIO.1					;comparator input on pin 6
;#define compOut GPIO.2				;comparator output on pin 5
;#define pot AN3								;potentiometer on pin 3
;----- Program
;dir GPIO 0b00111010						;set the port directions
	movlw	58
	banksel	TRISIO
	movwf	TRISIO
;CMCON0 = 0b00000011						;set up comparator
	movlw	3
	banksel	CMCON0
	movwf	CMCON0
;CMCON1 = 0b00000000
	clrf	CMCON1
;VRCON = 0b10001000						;set voltage reference to 2.5V
	movlw	136
	banksel	VRCON
	movwf	VRCON
;do
SysDoLoop_S1
;PWMOut 1, readAD(pot), 100  ;adjust 100 to change carrier freq
	movlw	3
	banksel	ADREADPORT
	movwf	ADREADPORT
	call	FN_READAD
	movlw	1
	movwf	PWMCHANNEL
	movf	READAD,W
	movwf	SOFTPWMDUTY
	movlw	100
	movwf	SOFTPWMCYCLES
	call	PWMOUT
;loop                    			;repeat in perpetuity
	goto	SysDoLoop_S1
SysDoLoop_E1
BASPROGRAMEND
	sleep
	goto	BASPROGRAMEND

;********************************************************************************

Delay_10US
D10US_START
	movlw	5
	movwf	DELAYTEMP
DelayUS0
	decfsz	DELAYTEMP,F
	goto	DelayUS0
	nop
	decfsz	SysWaitTemp10US, F
	goto	D10US_START
	return

;********************************************************************************

INITSYS
;OSCCON = OSCCON OR b'01110000'
	movlw	112
	banksel	OSCCON
	iorwf	OSCCON,F
;SET ADCON0.ADON OFF
	banksel	ADCON0
	bcf	ADCON0,ADON
;SET ADFM OFF
	bcf	ADCON0,ADFM
;ANSEL = 0
	banksel	ANSEL
	clrf	ANSEL
;CMCON0 = 7
	movlw	7
	banksel	CMCON0
	movwf	CMCON0
;GPIO = 0
	clrf	GPIO
	return

;********************************************************************************

PWMOUT
;For PWMDur = 1 to SoftPWMCycles
	clrf	PWMDUR
	movlw	1
	subwf	SOFTPWMCYCLES,W
	btfss	STATUS, C
	goto	SysForLoopEnd1
SysForLoop1
	incf	PWMDUR,F
;For DOPWM = 1 to 255
	clrf	DOPWM
SysForLoop2
	incf	DOPWM,F
;if SoftPWMDuty > DOPWM then
	movf	SOFTPWMDUTY,W
	subwf	DOPWM,W
	btfsc	STATUS, C
	goto	ELSE4_1
;if PWMChannel = 1 then set PWM_Out1 ON
	decf	PWMCHANNEL,W
	btfsc	STATUS, Z
	bsf	GPIO,0
;Else
	goto	ENDIF4
ELSE4_1
;if PWMChannel = 1 then set PWM_Out1 OFF
	decf	PWMCHANNEL,W
	btfsc	STATUS, Z
	bcf	GPIO,0
;end if
ENDIF4
;next
	movlw	255
	subwf	DOPWM,W
	btfss	STATUS, C
	goto	SysForLoop2
SysForLoopEnd2
;next
	movf	SOFTPWMCYCLES,W
	subwf	PWMDUR,W
	btfss	STATUS, C
	goto	SysForLoop1
SysForLoopEnd1
	return

;********************************************************************************

FN_READAD
;Set up for 8 bit
;Perform conversion
;LLReadAD
;Dim AllANSEL As Byte Alias ANSEL
;AllANSEL = 0
	banksel	ALLANSEL
	clrf	ALLANSEL
;ADTemp = ADReadPort + 1
	banksel	ADREADPORT
	incf	ADREADPORT,W
	movwf	ADTEMP
;Set C On
	bsf	STATUS,C
;Do
SysDoLoop_S2
;Rotate AllANSEL Left
	banksel	ALLANSEL
	rlf	ALLANSEL,F
;decfsz ADTemp,F
	banksel	ADTEMP
	decfsz	ADTEMP,F
;Loop
	goto	SysDoLoop_S2
SysDoLoop_E2
;SET ADCS1 OFF
	banksel	ANSEL
	bcf	ANSEL,ADCS1
;SET ADCS0 ON
	bsf	ANSEL,ADCS0
;SET ADCON0.CHS0 OFF
	banksel	ADCON0
	bcf	ADCON0,CHS0
;SET ADCON0.CHS1 OFF
	bcf	ADCON0,CHS1
;IF ADReadPort.0 On Then Set ADCON0.CHS0 On
	btfsc	ADREADPORT,0
	bsf	ADCON0,CHS0
;IF ADReadPort.1 On Then Set ADCON0.CHS1 On
	btfsc	ADREADPORT,1
	bsf	ADCON0,CHS1
;SET ADCON0.ADON ON
	bsf	ADCON0,ADON
;Wait AD_Delay
	movlw	2
	movwf	SysWaitTemp10US
	call	Delay_10US
;SET ADCON0.GO_DONE ON
	bsf	ADCON0,GO_DONE
;Wait While ADCON0.GO_DONE ON
SysWaitLoop1
	btfsc	ADCON0,GO_DONE
	goto	SysWaitLoop1
;SET ADCON0.ADON OFF
	bcf	ADCON0,ADON
;ANSEL = 0
	banksel	ANSEL
	clrf	ANSEL
;Write output
;ReadAD = ADRESH
	banksel	ADRESH
	movf	ADRESH,W
	movwf	READAD
	return

;********************************************************************************


 END
